@@ -641,39 +641,41 @@ gck_attribute_clear (GckAttribute *attr)
 
 /**
  * gck_attribute_free:
- * @attr: Attribute to free.
+ * @attr: (type Gck.Attribute): attribute to free
  *
  * Free an attribute and its allocated memory. These is usually
  * used with attributes that are allocated by gck_attribute_new()
  * or a similar function.
  **/
 void
-gck_attribute_free (GckAttribute *attr)
+gck_attribute_free (gpointer attr)
 {
+	GckAttribute *a = attr;
 	if (attr) {
-		attribute_clear (attr, g_realloc);
-		g_slice_free (GckAttribute, attr);
+		attribute_clear (a, g_realloc);
+		g_slice_free (GckAttribute, a);
 	}
 }
 
 /**
  * gck_attribute_equal:
- * @a: (type Gck.Attribute): first attribute to compare
- * @b: (type Gck.Attribute): second attribute to compare
+ * @attr1: (type Gck.Attribute): first attribute to compare
+ * @attr2: (type Gck.Attribute): second attribute to compare
  *
  * Compare two attributes. Useful with <code>GHashTable</code>.
  *
  * Returns: %TRUE if the attributes are equal.
  */
 gboolean
-gck_attribute_equal (gconstpointer a, gconstpointer b)
+gck_attribute_equal (gconstpointer attr1,
+                     gconstpointer attr2)
 {
-	const GckAttribute *aa = a;
-	const GckAttribute *ab = b;
+	const GckAttribute *aa = attr1;
+	const GckAttribute *ab = attr2;
 
-	if (!a && !b)
+	if (!aa && !ab)
 		return TRUE;
-	if (!a || !b)
+	if (!aa || !ab)
 		return FALSE;
 
 	if (aa->type != ab->type)
@@ -687,6 +689,29 @@ gck_attribute_equal (gconstpointer a, gconstpointer b)
 	return memcmp (aa->value, ab->value, aa->length) == 0;
 }
 
+/**
+ * gck_attribute_hash:
+ * @attr: (type Gck.Attribute): attribute to hash
+ *
+ * Hash an attribute for use in <code>GHashTable</code> keys.
+ *
+ * Returns: the hash code
+ */
+guint
+gck_attribute_hash (gconstpointer attr)
+{
+	const GckAttribute *a = attr;
+	const signed char *p, *e;
+	guint32 h = 5381;
+
+	h ^= _gck_ulong_hash (&a->type);
+
+	for (p = (signed char *)a->value, e = p + a->length; p != e; p++)
+		h = (h << 5) + h + *p;
+
+	return h;
+}
+
 /**
  * SECTION:gck-attributes
  * @title: GckAttributes
@@ -1281,6 +1306,7 @@ gboolean
 gck_attributes_find_string (GckAttributes *attrs, gulong attr_type, gchar **value)
 {
 	GckAttribute *attr;
+	gchar *string;
 
 	g_return_val_if_fail (value, FALSE);
 	g_return_val_if_fail (!attrs->locked, FALSE);
@@ -1288,7 +1314,10 @@ gck_attributes_find_string (GckAttributes *attrs, gulong attr_type, gchar **valu
 	attr = gck_attributes_find (attrs, attr_type);
 	if (!attr || gck_attribute_is_invalid (attr))
 		return FALSE;
-	*value = gck_attribute_get_string (attr);
+	string = gck_attribute_get_string (attr);
+	if (string == NULL)
+		return FALSE;
+	*value = string;
 	return TRUE;
 }
 
@@ -1346,21 +1375,22 @@ gck_attributes_ref (GckAttributes *attrs)
  * When all outstanding references are NULL, the array will be freed.
  */
 void
-gck_attributes_unref (GckAttributes *attrs)
+gck_attributes_unref (gpointer attrs)
 {
+	GckAttributes *attrs_ = attrs;
 	guint i;
 
-	if (!attrs)
+	if (!attrs_)
 		return;
 
-	if (g_atomic_int_dec_and_test (&attrs->refs)) {
-		g_return_if_fail (attrs->array);
-		g_return_if_fail (!attrs->locked);
-		for (i = 0; i < attrs->array->len; ++i)
-			attribute_clear (gck_attributes_at (attrs, i), attrs->allocator);
-		g_array_free (attrs->array, TRUE);
-		attrs->array = NULL;
-		g_slice_free (GckAttributes, attrs);
+	if (g_atomic_int_dec_and_test (&attrs_->refs)) {
+		g_return_if_fail (attrs_->array);
+		g_return_if_fail (!attrs_->locked);
+		for (i = 0; i < attrs_->array->len; ++i)
+			attribute_clear (gck_attributes_at (attrs_, i), attrs_->allocator);
+		g_array_free (attrs_->array, TRUE);
+		attrs_->array = NULL;
+		g_slice_free (GckAttributes, attrs_);
 	}
 }
 
