@@ -48,45 +48,16 @@
 enum {
 	PROP_0,
 	PROP_MODULE,
-	PROP_HANDLE,
-	PROP_INTERACTION
+	PROP_HANDLE
 };
 
 struct _GckSlotPrivate {
 	GckModule *module;
 	CK_SLOT_ID handle;
-
-	/* Changable data locked by mutex */
-	GMutex *mutex;
-	GTlsInteraction *interaction;
 };
 
 G_DEFINE_TYPE (GckSlot, gck_slot, G_TYPE_OBJECT);
 
-/* ----------------------------------------------------------------------------
- * HELPERS
- */
-
-static GckSession*
-make_session_object (GckSlot *self,
-                     GckSessionOptions options,
-                     CK_SESSION_HANDLE handle)
-{
-	GckSession *session;
-	GckModule *module;
-
-	g_return_val_if_fail (handle != 0, NULL);
-
-	module = gck_slot_get_module (self);
-
-	session = gck_session_from_handle (self, handle, options);
-	g_return_val_if_fail (session != NULL, NULL);
-
-	g_object_unref (module);
-
-	return session;
-}
-
 /* ----------------------------------------------------------------------------
  * OBJECT
  */
@@ -95,7 +66,6 @@ static void
 gck_slot_init (GckSlot *self)
 {
 	self->pv = G_TYPE_INSTANCE_GET_PRIVATE (self, GCK_TYPE_SLOT, GckSlotPrivate);
-	self->pv->mutex = g_mutex_new ();
 }
 
 static void
@@ -111,9 +81,6 @@ gck_slot_get_property (GObject *obj, guint prop_id, GValue *value,
 	case PROP_HANDLE:
 		g_value_set_ulong (value, gck_slot_get_handle (self));
 		break;
-	case PROP_INTERACTION:
-		g_value_take_object (value, gck_slot_get_interaction (self));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 		break;
@@ -139,35 +106,18 @@ gck_slot_set_property (GObject *obj, guint prop_id, const GValue *value,
 		g_assert (!self->pv->handle);
 		self->pv->handle = g_value_get_ulong (value);
 		break;
-	case PROP_INTERACTION:
-		gck_slot_set_interaction (self, g_value_get_object (value));
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, prop_id, pspec);
 		break;
 	}
 }
 
-static void
-gck_slot_dispose (GObject *obj)
-{
-	GckSlot *self = GCK_SLOT (obj);
-
-	gck_slot_set_interaction (self, NULL);
-
-	G_OBJECT_CLASS (gck_slot_parent_class)->dispose (obj);
-}
-
 static void
 gck_slot_finalize (GObject *obj)
 {
 	GckSlot *self = GCK_SLOT (obj);
 
-	g_assert (self->pv->interaction == NULL);
-
-	self->pv->handle = 0;
 	g_clear_object (&self->pv->module);
-	g_mutex_free (self->pv->mutex);
 
 	G_OBJECT_CLASS (gck_slot_parent_class)->finalize (obj);
 }
@@ -180,7 +130,6 @@ gck_slot_class_init (GckSlotClass *klass)
 
 	gobject_class->get_property = gck_slot_get_property;
 	gobject_class->set_property = gck_slot_set_property;
-	gobject_class->dispose = gck_slot_dispose;
 	gobject_class->finalize = gck_slot_finalize;
 
 	/**
@@ -201,17 +150,6 @@ gck_slot_class_init (GckSlotClass *klass)
 		g_param_spec_ulong ("handle", "Handle", "PKCS11 Slot ID",
 		                   0, G_MAXULONG, 0, G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
-	/**
-	 * GckSlot:interaction:
-	 *
-	 * Interaction object used to ask the user for pins when opening
-	 * sessions. Used if the session_options of the enumerator have
-	 * %GCK_SESSION_LOGIN_USER or %GCK_SESSION_AUTHENTICATE
-	 */
-	g_object_class_install_property (gobject_class, PROP_INTERACTION,
-		g_param_spec_object ("interaction", "Interaction", "Interaction asking for pins",
-		                     G_TYPE_TLS_INTERACTION, G_PARAM_READWRITE));
-
 	g_type_class_add_private (gobject_class, sizeof (GckSlotPrivate));
 }
 
@@ -942,62 +880,6 @@ gck_slot_has_flags (GckSlot *self, gulong flags)
 	return (info.flags & flags) != 0;
 }
 
-/**
- * gck_slot_get_interaction:
- * @self: the slot
- *
- * Get the interaction used when a pin is needed
- *
- * Returns: (transfer full) (allow-none): the interaction or %NULL
- */
-GTlsInteraction *
-gck_slot_get_interaction (GckSlot *self)
-{
-	GTlsInteraction *result = NULL;
-
-	g_return_val_if_fail (GCK_IS_SLOT (self), NULL);
-
-	g_mutex_lock (self->pv->mutex);
-
-		if (self->pv->interaction)
-			result = g_object_ref (self->pv->interaction);
-
-	g_mutex_unlock (self->pv->mutex);
-
-	return result;
-}
-
-/**
- * gck_slot_set_interaction:
- * @self: the slot
- * @interaction: (allow-none): the interaction or %NULL
- *
- * Set the interaction used when a pin is needed
- */
-void
-gck_slot_set_interaction (GckSlot *self,
-                          GTlsInteraction *interaction)
-{
-	GTlsInteraction *previous = NULL;
-
-	g_return_if_fail (GCK_IS_SLOT (self));
-	g_return_if_fail (interaction == NULL || G_IS_TLS_INTERACTION (interaction));
-
-	g_mutex_lock (self->pv->mutex);
-
-		if (interaction != self->pv->interaction) {
-			previous = self->pv->interaction;
-			self->pv->interaction = interaction;
-			if (interaction)
-				g_object_ref (interaction);
-		}
-
-	g_mutex_unlock (self->pv->mutex);
-
-	g_clear_object (&previous);
-	g_object_notify (G_OBJECT (self), "interaction");
-}
-
 /**
  * gck_slot_enumerate_objects:
  * @self: a #GckSlot to enumerate objects on
@@ -1052,62 +934,10 @@ gck_slots_enumerate_objects (GList *slots,
 	uri_data = gck_uri_data_new ();
 	uri_data->attributes = gck_attributes_ref (match);
 
-	return _gck_enumerator_new (slots, options, uri_data);
+	return _gck_enumerator_new_for_slots (slots, options, uri_data);
 }
 
 
-typedef struct OpenSession {
-	GckArguments base;
-	GTlsInteraction *interaction;
-	GckSlot *slot;
-	gulong flags;
-	gpointer app_data;
-	CK_NOTIFY notify;
-	gchar *password;
-	gboolean auto_login;
-	CK_SESSION_HANDLE session;
-} OpenSession;
-
-static CK_RV
-perform_open_session (OpenSession *args)
-{
-	GTlsInteraction *interaction;
-	CK_RV rv = CKR_OK;
-
-	/* Can be called multiple times */
-
-	/* First step, open session */
-	if (!args->session) {
-		rv = (args->base.pkcs11->C_OpenSession) (args->base.handle, args->flags,
-		                                         args->app_data, args->notify, &args->session);
-	}
-
-	if (rv != CKR_OK || !args->auto_login)
-		return rv;
-
-	/* Compatibility, hook into GckModule signals if no interaction set */
-	if (args->interaction)
-		interaction = g_object_ref (args->interaction);
-	else
-		interaction = _gck_interaction_new (args->slot);
-
-	rv = _gck_session_authenticate_token (args->base.pkcs11, args->session,
-	                                      args->slot, interaction, NULL);
-
-	g_object_unref (interaction);
-
-	return rv;
-}
-
-static void
-free_open_session (OpenSession *args)
-{
-	g_clear_object (&args->interaction);
-	g_clear_object (&args->slot);
-	g_assert (!args->password);
-	g_free (args);
-}
-
 /**
  * gck_slot_open_session:
  * @self: The slot ot open a session on.
@@ -1157,36 +987,12 @@ gck_slot_open_session_full (GckSlot *self,
                             GCancellable *cancellable,
                             GError **error)
 {
-	OpenSession args = { GCK_ARGUMENTS_INIT, 0,  };
-	GckSession *session = NULL;
-	GckModule *module = NULL;
-
-	g_object_ref (self);
-
-	module = gck_slot_get_module (self);
-
-	/* Open a new session */
-	args.slot = self;
-	args.app_data = app_data;
-	args.notify = notify;
-	args.password = NULL;
-	args.session = 0;
-	args.interaction = gck_slot_get_interaction (self);
-
-	args.auto_login = ((options & GCK_SESSION_LOGIN_USER) == GCK_SESSION_LOGIN_USER);
-
-	args.flags = pkcs11_flags | CKF_SERIAL_SESSION;
-	if ((options & GCK_SESSION_READ_WRITE) == GCK_SESSION_READ_WRITE)
-		args.flags |= CKF_RW_SESSION;
-
-	if (_gck_call_sync (self, perform_open_session, NULL, &args, cancellable, error))
-		session = make_session_object (self, options, args.session);
-
-	g_clear_object (&args.interaction);
-	g_object_unref (module);
-	g_object_unref (self);
-
-	return session;
+	return g_initable_new (GCK_TYPE_SESSION, cancellable, error,
+	                       "options", options,
+	                       "slot", self,
+	                       "opening-flags", pkcs11_flags,
+	                       "app-data", app_data,
+	                       NULL);
 }
 
 /**
@@ -1212,6 +1018,25 @@ gck_slot_open_session_async (GckSlot *self,
 	gck_slot_open_session_full_async (self, options, 0UL, NULL, NULL, cancellable, callback, user_data);
 }
 
+static void
+on_open_session_complete (GObject *source,
+                          GAsyncResult *result,
+                          gpointer user_data)
+{
+	GSimpleAsyncResult *res = G_SIMPLE_ASYNC_RESULT (user_data);
+	GError *error = NULL;
+	GObject *session;
+
+	session = g_async_initable_new_finish (G_ASYNC_INITABLE (source), result, &error);
+	if (session != NULL)
+		g_simple_async_result_set_op_res_gpointer (res, session, g_object_unref);
+	else
+		g_simple_async_result_take_error (res, error);
+
+	g_simple_async_result_complete (res);
+	g_object_unref (res);
+}
+
 /**
  * gck_slot_open_session_full_async: (skip)
  * @self: The slot to open a session on.
@@ -1238,26 +1063,24 @@ gck_slot_open_session_full_async (GckSlot *self,
                                   GAsyncReadyCallback callback,
                                   gpointer user_data)
 {
-	OpenSession *args;
+	GSimpleAsyncResult *res;
 
-	g_object_ref (self);
+	g_return_if_fail (GCK_IS_SLOT (self));
+	g_return_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable));
 
-	args =  _gck_call_async_prep (self, self, perform_open_session, NULL,
-	                              sizeof (*args), free_open_session);
+	res = g_simple_async_result_new (G_OBJECT (self), callback, user_data,
+	                                 gck_slot_open_session_full_async);
 
-	args->app_data = app_data;
-	args->notify = notify;
-	args->slot = g_object_ref (self);
-	args->interaction = gck_slot_get_interaction (self);
+	g_async_initable_new_async (GCK_TYPE_SESSION, G_PRIORITY_DEFAULT,
+	                            cancellable, on_open_session_complete,
+	                            g_object_ref (res),
+	                            "options", options,
+	                            "slot", self,
+	                            "opening-flags", pkcs11_flags,
+	                            "app-data", app_data,
+	                            NULL);
 
-	args->auto_login = ((options & GCK_SESSION_LOGIN_USER) == GCK_SESSION_LOGIN_USER);
-
-	args->flags = pkcs11_flags | CKF_SERIAL_SESSION;
-	if ((options & GCK_SESSION_READ_WRITE) == GCK_SESSION_READ_WRITE)
-		args->flags |= CKF_RW_SESSION;
-
-	_gck_call_async_ready_go (args, cancellable, callback, user_data);
-	g_object_unref (self);
+	g_object_unref (res);
 }
 
 /**
@@ -1271,25 +1094,21 @@ gck_slot_open_session_full_async (GckSlot *self,
  *
  * Returns: (transfer full): the new session or %NULL if an error occurs
  */
-GckSession*
-gck_slot_open_session_finish (GckSlot *self, GAsyncResult *result, GError **err)
+GckSession *
+gck_slot_open_session_finish (GckSlot *self, GAsyncResult *result, GError **error)
 {
-	GckSession *session = NULL;
+	GSimpleAsyncResult *res;
 
-	g_object_ref (self);
+	g_return_val_if_fail (GCK_IS_SLOT (self), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+	g_return_val_if_fail (g_simple_async_result_is_valid (result, G_OBJECT (self),
+	                      gck_slot_open_session_full_async), NULL);
 
-	{
-		OpenSession *args;
+	res = G_SIMPLE_ASYNC_RESULT (result);
+	if (g_simple_async_result_propagate_error (res, error))
+		return NULL;
 
-		if (_gck_call_basic_finish (result, err)) {
-			args = _gck_call_arguments (result, OpenSession);
-			session = make_session_object (self, args->flags, args->session);
-		}
-	}
-
-	g_object_unref (self);
-
-	return session;
+	return g_object_ref (g_simple_async_result_get_op_res_gpointer (res));
 }
 
 /**
